

from setuptools import setup, find_packages


setup(
    name="pekutko-serializer",
    version="0.0.8",
    description="Dmitry Pekutko, gruop 053504, format cenverter (Serializer)",
    author="Dmitry Pekutko",
    author_email="dkefir03@gmail.com",
    python_requires='>3.8.0',
    packages=find_packages(),
    url="https://github.com/DimaPekutko/ISP-2022-053504/",
)